@@ -1,12 +1,15 @@
 package MaxMind::DB::Writer::Tree;
-$MaxMind::DB::Writer::Tree::VERSION = '0.080000';
+
 use strict;
 use warnings;
 use namespace::autoclean;
 use autodie;
 
+our $VERSION = '0.100001';
+
 use IO::Handle;
-use Math::Int128 0.06 qw( uint128 );
+use Math::Int64 0.51;
+use Math::Int128 0.21 qw( uint128 );
 use MaxMind::DB::Common 0.031003 qw(
     DATA_SECTION_SEPARATOR
     METADATA_MARKER
@@ -25,13 +28,7 @@ use MooseX::StrictConstructor;
 
 use XSLoader;
 
-XSLoader::load(
-    __PACKAGE__,
-    exists $MaxMind::DB::Writer::Tree::{VERSION}
-        && ${ $MaxMind::DB::Writer::Tree::{VERSION} }
-    ? ${ $MaxMind::DB::Writer::Tree::{VERSION} }
-    : '42'
-);
+XSLoader::load( __PACKAGE__, $VERSION );
 
 has ip_version => (
     is       => 'ro',
@@ -332,7 +329,7 @@ MaxMind::DB::Writer::Tree - Tree representing a MaxMind DB database in memory -
 
 =head1 VERSION
 
-version 0.080000
+version 0.100001
 
 =head1 SYNOPSIS
 
